#!/usr/bin/env python
# Copyright (C) 2010 W. Trevor King <wking@drexel.edu>
#
# This file is part of SiteCorePy.
#
# SiteCorePy is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# SiteCorePy is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SiteCorePy.  If not, see <http://www.gnu.org/licenses/>.

"""A Python_ interface to SiteCore_ via WebDriver_.

Because I like my wrists too much to enter data by hand.

Note that WebDriver is currently being merged into Selenium_.

.. _Python: http://www.python.org/
.. _SiteCore: http://www.sitecore.net/
.. _WebDriver: http://code.google.com/p/selenium/
.. _Selenium: http://code.google.com/p/selenium/
"""

import getpass
import logging
import time

try:
    from selenium.firefox.webdriver import WebDriver
except ImportError, e:
    WebDriver = e  # allow some functionality without selenium

def get_logger(verbose=0):
    verbosities = [  # in order of decreasing severity
        logging.CRITICAL,
        logging.ERROR,
        logging.WARN,
        logging.INFO,
        logging.DEBUG,
        ]
    logger = logging.getLogger('sitecore')
    if verbose >= len(verbosities):
        verbose = -1
    logger.setLevel(verbosities[verbose])
    # create console handler and set the same level
    ch = logging.StreamHandler()
    ch.setLevel(verbosities[verbose])
    # create formatter
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # add formatter to ch
    ch.setFormatter(formatter)
    # add ch to logger
    logger.addHandler(ch)
    return logger


class SiteCoreConnection (object):
    def __init__(self, url, verbose=0):
        if isinstance(WebDriver, ImportError):
            raise WebDriver  # *now* require sitecore
        self.url = url
        self.logger = get_logger(verbose)
        self.wait_time = 5

    def start(self):
        'Start a single Firefox instance'
        self.logger.info('starting Firefox')
        self.w = WebDriver()
        self.w.get(self.url)

    def stop(self):
        'Stop the Firefox instance'
        self.logger.info('stopping Firefox')
        self.w.quit()

    def login(self, username=None, password=None):
        if username == None:
            username = getpass.getpass('username: ')
        if password == None:
            password = getpass.getpass('password: ')
        self.logger.info('logging in as %s' % username)
        uname = self.w.find_element_by_id('Login_UserName')
        uname.send_keys(username)
        pword = self.w.find_element_by_id('Login_Password')
        pword.send_keys(password)
        submit = self.w.find_element_by_id('Login_Login')
        submit.click()
        del(username)
        del(password)

        time.sleep(self.wait_time)

    def find_nav_section(self, section):
        self.logger.info('finding nav section %s' % section)
        nav_bar = self.w.find_element_by_xpath("//tr[@id='ContentTreeHolder']")
        link = nav_bar.find_element_by_link_text(section)
        link_id = link.get_attribute('id')
        img_id = link_id.replace('_Node_', '_Glyph_')
        img = nav_bar.find_element_by_id(img_id)
        div = nav_bar.find_element_by_xpath("//img[@id='%s']/.." % img_id)
        return (div, img, link)

    def expand_nav_section(self, section):
        div,img,link = self.find_nav_section(section)
        src = img.get_attribute('src')
        if 'collapse' in src:
            self.logger.info('nav section %s already expanded' % section)
            return  # already expanded, no further action needed.
            self.logger.info('nav section %s already expanded' % section)
        assert 'expand' in src, src
        self.logger.info('expanding nav section %s' % section)
        img.click()
        time.sleep(self.wait_time)

    def open_nav_section(self, section):
        div,img,link = self.find_nav_section(section)
        self.logger.info('opening nav section %s' % section)
        link.click()
        time.sleep(self.wait_time)

    def lock_section(self):
        self.w.find_element_by_link_text('Lock and Edit').click()
        time.sleep(self.wait_time)

    def save_section(self):
        self.logger.info('saving current section')
        self.w.find_element_by_link_text('Save').click()
        time.sleep(self.wait_time)

    def publish_section(self, message):
        self.save_section()
        self.logger.info('publishing current section')
        self.w.find_element_by_link_text('Review').click()
        time.sleep(self.wait_time)
        self.w.find_element_by_link_text('Approve for publication').click()
        time.sleep(self.wait_time)
        windows = self.w.get_window_handles()
        current_window = self.w.get_current_window_handle()
        self.logger.info('handling popup %s (from %s)'
                         % (current_window, windows))
        #self.w.switch_to_window(name)
        a = self.w.switch_to_active_element()
        a.send_keys(message+'\n')

    def find_field(self, field):
        self.logger.info('finding %s field' % field)
        # found id start by manually focusing in the element and running
        #   >>> a = s.w.switch_to_active_element()
        #   >>> a.get_attribute('id')
        # FIELD*, but the number * changes between sessions/versions
        fields = []
        for tag in ['input', 'textarea']:
            fields.extend([f for f in self.w.find_elements_by_tag_name(tag)
                           if f.get_attribute('id').startswith('FIELD')])
        match = False
        for f in fields:
            granddad = f.find_element_by_xpath('/../..')
            if granddad.get_text().startswith(field):
                match = True
                break
        if match != True:
            raise KeyError(field)
        return (granddad, f)
